$set 8  #slattach

$ #_user_unkn Original Message:(slattach: tty_lock: UUCP user %s unknown!\n)
# slattach: tty_lock: UUCP Benutzer %s unbekannt!\n

$ #_drop Original Message:(slattach: tty_hangup(DROP): %s\n)
# slattach: tty_hangup(BEENDEN): %s\n

$ #_raise Original Message:(slattach: tty_hangup(RAISE): %s\n)
# slattach: tty_hangup(STARTEN): %s\n

$ #_cant_state Original Message:(slattach: tty_open: cannot get current state!\n)
# slattach: tty_open: kann aktuellen Zustand nicht bestimmen!\n

$ #_cant_disc Original Message:(slattach: tty_open: cannot get current line disc!\n)
# slattach: tty_open: kann aktuelle Leitungsdisziplin nicht bestimmen!\n

$ #_cant_raw Original Message:(slattach: tty_open: cannot set RAW mode!\n)
# slattach: tty_open: kann RAW-Modus nicht setzen!\n

$ #_cant_bps Original Message:(slattach: tty_open: cannot set %s bps!\n)
# slattach: tty_open: kann Baudrate %s nicht setzen!\n

$ #_cant_8n1 Original Message:(slattach: tty_open: cannot set 8N1 mode!\n)
# slattach: tty_open: kann 8N1-Modus nicht setzen!\n

$ #_usage1 Original Message:(Usage: slattach [-ehlLmnqv] [-c cmd] [-s speed] [-p protocol] tty | -\n)
# Benutzung: slattach [-ehlLmnqv] [-c Bef] [-s Geschw] [-p Proto] tty | -\n

$ #_unsupp Original Message:(slattach: unsupported protocol %s\n)
# slattach: Protokoll nicht unterstützt: %s\n

$ #_started Original Message:(%s started)
# %s hat begonnen

$ #_on Original Message:( on %s)
#  auf %s

$ #_iface Original Message:( interface %s\n)
#  Schnittstelle %s\n

$ #_locked Original Message:(slattach: /dev/%s already locked\n)
# slattach: /dev/%s bereits benutzt\n

$ #_cant_pid Original Message:(slattach: cannot write PID to lock file\n)
# slattach: kann PID nicht ins lockfile schreiben\n
