$set 3  #arp

$ #_hostname Original Message:(arp: need host name\n)
# arp: need host name\n

$ #_no_arp Original Message:(No ARP entry for %s\n)
# No ARP entry for %s\n

$ #_need_hw Original Message:(arp: need hardware address\n)
# arp: need hardware address\n

$ #_invalidhw Original Message:(arp: invalid hardware address\n)
# arp: invalid hardware address\n

$ #_cant_open Original Message:(arp: cannot open etherfile %s !\n)
# arp: cannot open etherfile %s !\n

$ #_formaterr Original Message:(arp: format error on line %u of etherfile %s !\n)
# arp: format error on line %u of etherfile %s !\n

$ #_cant_set Original Message:(arp: cannot set entry on line %u of etherfile %s !\n)
# arp: cannot set entry on line %u of etherfile %s !\n

$ #_address Original Message:(Address\t\tHW type\t\tHW address\t     Flags Mask\t\t   Iface\n)
# Address\t\tHW type\t\tHW address\t     Flags Mask\t\t   Iface\n

$ #_wrong_af Original Message:(arp: %s: kernel only supports 'inet'.\n)
# arp: %s: kernel only supports 'inet'.\n

$ #_sum Original Message:(Entries: %d\tSkiped: %d\tFound: %d\n)
# Entries: %d\tSkiped: %d\tFound: %d\n

$ #_usage1 Original Message:(Usage: arp [-vn] [-H type] [-i if] -a [hostname]\n)
# Usage: arp [-vn] [-H type] [-i if] -a [hostname]\n

$ #_usage2 Original Message:(       arp [-v] [-i if] -d hostname [pub]\n)
#        arp [-v] [-i if] -d hostname [pub]\n

$ #_usage3 Original Message:(       arp [-v] [-H type] [-i if] -s hostname hw_addr [temp] [trail]\n)
#        arp [-v] [-H type] [-i if] -s hostname hw_addr [temp] [trail]\n

$ #_usage4 Original Message:(       arp [-v] [-H type] [-i if] -s hostname hw_addr [netmask nm] pub [trail]\n)
#        arp [-v] [-H type] [-i if] -s hostname hw_addr [netmask nm] pub [trail]\n

$ #_usage5 Original Message:(       arp [-vn] [-H type] [-i if] -f filename\n)
#        arp [-vn] [-H type] [-i if] -f filename\n

$ #_hw_not_supp Original Message:(%s: hardware type not supported!\n)
# %s: hardware type not supported!\n

$ #_fam_not_supp Original Message:(%s: address family not supported!\n)
# %s: address family not supported!\n

$ #_unkn_addr Original Message:(arp: %s: unknown address family.\n)
# arp: %s: unknown address family.\n

$ #_unkn_hw Original Message:(arp: %s: unknown hardware type.\n)
# arp: %s: unknown hardware type.\n

$ #_none Original Message:(arp: in %d entries no match found.\n)
# arp: in %d entries no match found.\n

$ #_wrong_hw Original Message:(arp: %s: hardware type without ARP support.\n)
# arp: %s: hardware type without ARP support.\n

$ #_einv_pub Original Message:(Probably destination is reached via ARP Interface. See arp(8)\n)
# Probably destination is reached via ARP Interface. See arp(8)\n

$ #_einv_nopub Original Message:(Probably destination is on different Interface. See arp(8)\n)
# Probably destination is on different Interface. See arp(8)\n
